struct Web_Fragment;

struct Web_Segment {
    Web_Segment();
    void get_endpoints(Vec2 *p0_return, Vec2 *p1_return);
    Vec2 find_closest_point(Vec2 target, float *t_return = NULL);
    float get_wetness(float t);

    Web_Fragment *fragment_a;
    Web_Fragment *fragment_b;
    Spring_Force force;
    int player_number;
    int segment_id;
};

enum Spider_Motion_Mode {
    MOTION_MODE_RESPAWNING,
    MOTION_MODE_WALKING,
    MOTION_MODE_FALLING,
    MOTION_MODE_CLENCHING
};

enum Fly_Motion_Mode {
    FLY_FLYING,
    FLY_STUCK,
    FLY_STEWING,
    FLY_DONE
};

enum Entity_Type {
    ENTITY_TYPE_WEB_FRAGMENT,
    ENTITY_TYPE_SPIDER,
    ENTITY_TYPE_DUMMY,
    ENTITY_TYPE_FLY
};

struct Entity {
    Entity();
    virtual ~Entity() {}

    void set_position(Vec2 position);
    void set_mass(float mass);

    Entity_Type type;
    Rigid_Body rb;
    int player_number;
};

struct Web_Fragment : public Entity {
    Web_Fragment();
    float wetness;
};




struct Fly : public Entity {
    Fly();
    ~Fly();

    void update();
    void yes_antigravity();
    void no_antigravity();

    Fly_Motion_Mode motion_mode;
    float spawn_time;
    float time_to_be_released;

    float stew_time_begin;
    float stew_time_end;

    Vec2 spawn_position;
    float amplitude1, amplitude2;
    float period1, period2;

    Spring_Force *web_tie;
    Constant_Force *antigravity;
};


struct Spider : public Entity {
    Spider(int player_number);

    void update();
    void respawn();

    Vec2 LeftStick, RightStick;

    Web_Segment *spin_segment;
    Web_Segment *walk_segment;

    Spider_Motion_Mode motion_mode;

    Constant_Force antigravity;

    Constant_Force clench_force;
    Spring_Force clench_tie;
    bool clench_forces;

    Constant_Force spin_force;
    Spring_Force spin_tie;
    bool spin_forces;

    Constant_Force *tug_force;
    Auto_Array <Web_Segment *> segments_i_cannot_climb;

    float respawn_time;
    float speed;

    bool wants_falling;
    bool wants_clenching;
    bool wants_spinning;
    bool wants_webstick;
};

